#Created by: Melissa Trayhan, Rochester Center For Health Informatics
#This file reads in trimmed, sorted claims data and creates a nework based on the sliding window method

#Use sorted.pl before running this script to get the claims data in the correct format and sorted for the correct network to be created.
#data format:
#Patient id [0], Date [1], Organization NPI [2], Attending NPI [3], Operating NPI [4]

#!/usr/bin/perl
use strict;
use warnings;
use Date::Calc qw(Delta_Days);

#specify claims data and check if it is a file
my $file = $ARGV[0] or die "Please state the CSV filename as arg 1\n";
unless (-f $file) {
  die "Not a valid file! Try again\n";
}

#select which type of network to create: organization to organization, attending provider to attening provider, or operating provider to operating provider
my $npi_type = $ARGV[1] or die "Please state the npi type as arg 2, use ORG for organization, PRV for provider or OPR for operating\n";
my $npi_col;

if ($npi_type eq "ORG" || $npi_type eq "org") {
  $npi_col = 2;
}
elsif ($npi_type eq "PRV" || $npi_type eq "prv") {
  $npi_col = 3;
}
elsif ($npi_type eq "OPR" || $npi_type eq "opr") {
  $npi_col = 4;
}
else {
  die "Please state the npi type as arg 2, use ORG for organization, PRV for provider or OPR for operating\n";
}

#select the window size desired to create the network, options are 30, 60, 90, 190 and 365 days
my $window_size = $ARGV[2] or die "Please state the window size (30, 60, 90, 180, or 365) as arg 3\n";

if ($window_size == 30) {} elsif ($window_size == 60) {} elsif ($window_size == 90) {} elsif ($window_size == 180) {} elsif ($window_size == 365) {}
else {
  die "Please state the window size (30, 60, 90, 180, or 365) as arg 3, you supplied: $window_size\n";
}


#open the sorted claims file and import into an array of arrays
open(my $data, '<', $file) or die "Could not open '$file' $!\n";
my @sorted;

while (my $line = <$data>) {
  chomp $line;
  my @temp = split "," , $line;
  push @sorted, \@temp;
}

close $data;


#calculate organization/provider pairs. Defined by 2 different providers that see the same patient within the given timeframe/window size (directed)

my $counter = 0;
my $patientid = 0;
my $date = 0;

my @ref_matrix;

for (my $row = 0; $row < @sorted; $row++ ) {
  $patientid = $sorted[$row][0];
  $date = $sorted[$row][1];
  my ($y1) = $date =~ /^(\d\d\d\d)/;
  my ($m1) = $date =~ /^\d\d\d\d(\d\d)/;
  my ($d1) = $date =~ /^\d\d\d\d\d\d(\d\d)/;
  $counter = $row + 1;
  while ($counter < @sorted && $sorted[$counter][0] eq $patientid ) {
    my $nextdate = $sorted[$counter][1];
    my ($y2) = $nextdate =~ /^(\d\d\d\d)/;
    my ($m2) = $nextdate =~ /^\d\d\d\d(\d\d)/;
    my ($d2) = $nextdate =~ /^\d\d\d\d\d\d(\d\d)/;
    my $Dd = Delta_Days($y1,$m1,$d1,$y2,$m2,$d2);
    if ($Dd <= $window_size) {
      if ($sorted[$row][$npi_col] && $sorted[$counter][$npi_col]) {
        if ($sorted[$row][$npi_col] != $sorted[$counter][$npi_col]) {   #remove self referrals 
          my @temp = "$sorted[$row][$npi_col],$sorted[$counter][$npi_col] : $sorted[$row][0]";
          push @ref_matrix, @temp;
        }
      }
    }
    $counter++;    
  }
}


#total relationship pairs: count the instances of a relationship between two npis. Remove any that occur for less than 11 patients

#open results file
my $results_file = "visit_counts_sliding_$ARGV[1]_$window_size.csv";

open(my $refs, '>', $results_file) or die "Could not open '$results_file' $!\n";

my $total_visit_count = 1;  #the total number of visits
my $unique_patient_count = 1; #the number of unique patients
my $npi1 = "";
my $npi2 = "";

@ref_matrix = sort @ref_matrix;

my @prev = split(" : ", $ref_matrix[0]);
my $x = 1;

#sum up the pair counts to find both total visits and unique patients
while ($x < @ref_matrix) {  
  my @temp = split(":", $ref_matrix[$x]);
  if ($temp[0] eq $prev [0] ) { #if the npi pair is the same, add to the visit count
    $total_visit_count++;
    if ($temp[1] != $prev[1]) { #if its a different patient add to the patient count
      $unique_patient_count++;
    }
    $x++;
    @prev = @temp;
  }
  else {
    if ($unique_patient_count> 10) {  #if more than 10 patients saw these two npis, add to the output file   *11 Patient minimums are required by CMS/HIPAA
      print $refs "$prev[0],$unique_patient_count,$total_visit_count\n";
    }
    $total_visit_count = 1;
    $unique_patient_count = 1;
    @prev = @temp;
    $x++;
  }
}

close $refs;


exit;